package com.yunfeiyun.agmp.common.web.system.service.impl;

import com.yunfeiyun.agmp.common.constant.ErrorCode;
import com.yunfeiyun.agmp.common.constant.UserConstants;
import com.yunfeiyun.agmp.common.exception.BizException;
import com.yunfeiyun.agmp.common.framework.web.service.ISysMenuService;
import com.yunfeiyun.agmp.common.framework.web.service.ISysRoleService;
import com.yunfeiyun.agmp.common.web.system.domain.SysRoleDept;
import com.yunfeiyun.agmp.common.web.system.domain.SysRoleMenu;
import com.yunfeiyun.agmp.common.web.system.mapper.SysRoleDeptMapper;
import com.yunfeiyun.agmp.common.web.system.mapper.SysRoleMapper;
import com.yunfeiyun.agmp.common.web.system.mapper.SysRoleMenuMapper;
import com.yunfeiyun.agmp.common.web.system.mapper.SysUserRoleMapper;
import com.yunfeiyun.agmp.common.core.domain.entity.SysRole;
import com.yunfeiyun.agmp.common.core.domain.entity.SysUser;
import com.yunfeiyun.agmp.common.core.domain.entity.SysUserRole;
import com.yunfeiyun.agmp.common.core.domain.model.IBaseMenu;
import com.yunfeiyun.agmp.common.core.domain.model.IBaseRole;
import com.yunfeiyun.agmp.common.core.domain.model.IUserRole;
import com.yunfeiyun.agmp.common.utils.SecurityUtils;
import com.yunfeiyun.agmp.common.utils.StringUtils;
import com.yunfeiyun.agmp.common.utils.spring.SpringUtils;
import com.yunfeiyun.agmp.common.utils.uuid.Uuids;
import com.yunfeiyun.agmp.common.web.system.service.ISysMenuConfigService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;
import java.util.stream.Collectors;

/**
 * 角色 业务层处理
 * 
 * @author 杨晓辉
 */
@Service
public class SysRoleServiceImpl implements ISysRoleService
{
    private static final Logger log = LoggerFactory.getLogger(SysRoleServiceImpl.class);
    @Autowired
    private SysRoleMapper roleMapper;

    @Autowired
    private SysRoleMenuMapper roleMenuMapper;

    @Autowired
    private SysUserRoleMapper userRoleMapper;

    @Autowired
    private SysRoleDeptMapper roleDeptMapper;

    @Autowired
    private ISysMenuService menuService;

    @Autowired
    private ISysMenuConfigService menuConfigService;

    /**
     * 根据条件分页查询角色数据
     * 
     * @param role 角色信息
     * @return 角色数据集合信息
     */
    @Override
    //@DataScope(deptAlias = "d")  禁用数据权限，请知晓
    public List<IBaseRole> selectRoleList(IBaseRole role)
    {
        return roleMapper.selectRoleList(role);
    }

    /**
     * 根据用户ID查询角色
     * 
     * @param userId 用户ID
     * @return 角色列表
     */
    @Override
    public List<IBaseRole> selectRolesByUserId(String userId)
    {
        List<IBaseRole> userRoles = roleMapper.selectRolePermissionByUserId(userId);
        List<IBaseRole> roles = selectRoleAll();
        for (IBaseRole role : roles)
        {
            for (IBaseRole userRole : userRoles)
            {
                if (role.getRoleId().equals(userRole.getRoleId()))
                {
                    role.setFlag(true);
                    break;
                }
            }
        }
        return roles;
    }

    /**
     * 根据用户ID查询权限
     * 
     * @param userId 用户ID
     * @return 权限列表
     */
    @Override
    public Set<String> selectRolePermissionByUserId(String userId)
    {
        List<IBaseRole> perms = roleMapper.selectRolePermissionByUserId(userId);
        Set<String> permsSet = new HashSet<>();
        for (IBaseRole perm : perms)
        {
            if (StringUtils.isNotNull(perm))
            {
                permsSet.addAll(Arrays.asList(perm.getRoleKey().trim().split(",")));
            }
        }
        return permsSet;
    }

    /**
     * 查询所有角色
     * 
     * @return 角色列表
     */
    @Override
    public List<IBaseRole> selectRoleAll()
    {
        return SpringUtils.getAopProxy(this).selectRoleList(new SysRole());
    }

    /**
     * 根据用户ID获取角色选择框列表
     * 
     * @param userId 用户ID
     * @return 选中角色ID列表
     */
    @Override
    public List<Long> selectRoleListByUserId(Long userId)
    {
        return roleMapper.selectRoleListByUserId(userId);
    }

    /**
     * 通过角色ID查询角色
     * 
     * @param roleId 角色ID
     * @return 角色对象信息
     */
    @Override
    public SysRole selectRoleById(String roleId)
    {
        return roleMapper.selectRoleById(roleId);
    }

    /**
     * 校验角色名称是否唯一
     * 
     * @param role 角色信息
     * @return 结果
     */
    @Override
    public boolean checkRoleNameUnique(IBaseRole role)
    {
        String roleId = StringUtils.isNull(role.getRoleId()) ? "-1" : role.getRoleId();
        SysRole info = roleMapper.checkRoleNameUnique(role.getRoleName());
        if (StringUtils.isNotNull(info) && !info.getRoleId().equals(roleId.toString()))
        {
            return UserConstants.NOT_UNIQUE;
        }
        return UserConstants.UNIQUE;
    }

    /**
     * 校验角色权限是否唯一
     * 
     * @param role 角色信息
     * @return 结果
     */
    @Override
    public boolean checkRoleKeyUnique(IBaseRole role)
    {
        String roleId = StringUtils.isNull(role.getRoleId()) ? "-1" : role.getRoleId();
        SysRole info = roleMapper.checkRoleKeyUnique(role.getRoleKey());
        if (StringUtils.isNotNull(info) && !info.getRoleId().equals(roleId.toString()))
        {
            return UserConstants.NOT_UNIQUE;
        }
        return UserConstants.UNIQUE;
    }

    /**
     * 校验角色是否允许操作
     * 
     * @param role 角色信息
     */
    @Override
    public void checkRoleAllowed(IBaseRole role)
    {
        if (StringUtils.isNotNull(role.getRoleId()) && role.isAdmin())
        {
            throw new BizException(ErrorCode.ACCESS_FORBIDDEN.getCode(),"不允许操作超级管理员角色");
        }
    }

    /**
     * 校验角色是否有数据权限
     * 
     * @param roleId 角色id
     */
    @Override
    public void checkRoleDataScope(String roleId)
    {
        if (!SysUser.isAdmin(SecurityUtils.getUserId()))
        {
            SysRole role = new SysRole();
            role.setRoleId(roleId);
            List<IBaseRole> roles = SpringUtils.getAopProxy(this).selectRoleList(role);
            if (StringUtils.isEmpty(roles))
            {
                throw new BizException(ErrorCode.ACCESS_FORBIDDEN.getCode(),"没有权限访问角色数据！");
            }
        }
    }

    /**
     * 通过角色ID查询角色使用数量
     * 
     * @param roleId 角色ID
     * @return 结果
     */
    @Override
    public int countUserRoleByRoleId(String roleId)
    {
        return userRoleMapper.countUserRoleByRoleId(roleId);
    }

    /**
     * 新增保存角色信息
     * 
     * @param role 角色信息
     * @return 结果
     */
    @Override
    @Transactional
    public int insertRole(IBaseRole role)
    {
        // 新增角色信息
        role.setRoleId(role.getId());
        role.setRoleDataScope("1");
        role.setRoleDelstatus("0");
        roleMapper.insertRole(role);
        return insertRoleMenu(role);
    }

    /**
     * 修改保存角色信息
     * 
     * @param role 角色信息
     * @return 结果
     */
    @Override
    @Transactional
    public int updateRole(IBaseRole role)
    {
        // 修改角色信息
        roleMapper.updateRole(role);
        // 删除角色与菜单关联
        roleMenuMapper.deleteRoleMenuByRoleId(role.getRoleId());
        return insertRoleMenu(role);
    }

    /**
     * 修改角色状态
     * 
     * @param role 角色信息
     * @return 结果
     */
    @Override
    public int updateRoleStatus(IBaseRole role)
    {
        return roleMapper.updateRole(role);
    }

    /**
     * 修改数据权限信息
     * 
     * @param role 角色信息
     * @return 结果
     */
    @Override
    @Transactional
    public int authDataScope(IBaseRole role)
    {
        // 修改角色信息
        roleMapper.updateRole(role);
        // 删除角色与部门关联
        roleDeptMapper.deleteRoleDeptByRoleId(role.getRoleId());
        // 新增角色和部门信息（数据权限）
        return insertRoleDept(role);
    }

    /**
     * 新增角色菜单信息
     * 
     * @param role 角色对象
     */
    public int insertRoleMenu(IBaseRole role)
    {
        int rows = 1;
        List<String> cfgIds = menuConfigService.selectMenuConfigIds();
        // 新增用户与角色管理
        List<SysRoleMenu> list = new ArrayList<SysRoleMenu>();
        for (String menuId : role.getMenuIds())
        {
            if(!cfgIds.contains(menuId)){
                SysRoleMenu rm = new SysRoleMenu();
                rm.setRoleMenuId(Uuids.getId());
                rm.setRoleId(role.getRoleId());
                rm.setMenuId(menuId);
                list.add(rm);
            }
        }
        if (list.size() > 0)
        {
            rows = roleMenuMapper.batchRoleMenu(list);
            int rowsApi = insertRoleApi(role, list );
            log.info("插入角色id=[" + role.getRoleId() + "] 菜单=[" +  String.join(";", role.getMenuIds()) + "]    执行行数=[" +  rows + "]  执行扩展接口数=[" + rowsApi + "]");
        }

        return rows;
    }


    /**
     * 新增角色扩展API信息
     *
     * @param roleMenus 角色对象
     */
    public int insertRoleApi(IBaseRole role , List<SysRoleMenu> roleMenus)
    {
        int rows = 1;
        if(roleMenus.size() > 0){
            List<String> list = roleMenus.stream().map((v)->v.getMenuId()).collect(Collectors.toList());
            if(list.size() > 0 ){
                List<IBaseMenu> apiMenus =  menuService.buildMenuApiSelect(list);
                List<String> apiMenuList = apiMenus.stream().map(IBaseMenu::getMenuId).collect(Collectors.toList());
                if (apiMenuList.size() > 0)
                {
                    List<SysRoleMenu> roleList = new ArrayList<SysRoleMenu>();
                    for (String menuId : apiMenuList)
                    {
                        SysRoleMenu rm = new SysRoleMenu();
                        rm.setRoleMenuId(Uuids.getId());
                        rm.setRoleId(role.getRoleId());
                        rm.setMenuId(menuId);
                        roleList.add(rm);
                    }
                    if (roleList.size() > 0)
                    {
                        rows = roleMenuMapper.batchRoleMenu(roleList);
                    }
                    log.info("插入角色id=[" + role.getRoleId() + "] 隐藏接口=[" +  String.join(";", apiMenuList) + "]   执行扩展接口数=[" + rows + "]");
                    return rows;
                }
            }
        }
        return 0;
    }


    /**
     * 新增角色部门信息(数据权限)
     *
     * @param role 角色对象
     */
    public int insertRoleDept(IBaseRole role)
    {
        int rows = 1;
        // 新增角色与部门（数据权限）管理
        List<SysRoleDept> list = new ArrayList<SysRoleDept>();
        for (String deptId : role.getDeptIds())
        {
            SysRoleDept rd = new SysRoleDept();
            rd.setRoleId(role.getRoleId());
            rd.setDeptId(deptId);
            list.add(rd);
        }
        if (list.size() > 0)
        {
            rows = roleDeptMapper.batchRoleDept(list);
        }
        return rows;
    }

    /**
     * 通过角色ID删除角色
     * 
     * @param roleId 角色ID
     * @return 结果
     */
    @Override
    @Transactional
    public int deleteRoleById(String roleId)
    {
        // 删除角色与菜单关联
        roleMenuMapper.deleteRoleMenuByRoleId(roleId);
        // 删除角色与部门关联
        roleDeptMapper.deleteRoleDeptByRoleId(roleId);
        return roleMapper.deleteRoleById(roleId);
    }

    /**
     * 批量删除角色信息
     * 
     * @param roleIds 需要删除的角色ID
     * @return 结果
     */
    @Override
    @Transactional
    public int deleteRoleByIds(String[] roleIds)
    {
        for (String roleId : roleIds)
        {
            checkRoleAllowed(new SysRole(roleId));
            checkRoleDataScope(roleId);
            SysRole role = selectRoleById(roleId);
            if (countUserRoleByRoleId(roleId) > '0')
            {
                throw new BizException(ErrorCode.ACCESS_FORBIDDEN.getCode(),String.format("%1$s已分配,不能删除", role.getRoleName()));
            }
        }
        // 删除角色与菜单关联
        roleMenuMapper.deleteRoleMenu(roleIds);
        // 删除角色与部门关联
        //roleDeptMapper.deleteRoleDept(roleIds);
        return roleMapper.deleteRoleByIds(roleIds);
    }

    /**
     * 取消授权用户角色
     * 
     * @param userRole 用户和角色关联信息
     * @return 结果
     */
    @Override
    public int deleteAuthUser(IUserRole userRole)
    {
        return userRoleMapper.deleteUserRoleInfo(userRole);
    }

    /**
     * 批量取消授权用户角色
     * 
     * @param roleId 角色ID
     * @param userIds 需要取消授权的用户数据ID
     * @return 结果
     */
    @Override
    public int deleteAuthUsers(String roleId, String[] userIds)
    {
        return userRoleMapper.deleteUserRoleInfos(roleId, userIds);
    }

    /**
     * 批量选择授权用户角色
     * 
     * @param roleId 角色ID
     * @param userIds 需要授权的用户数据ID
     * @return 结果
     */
    @Override
    public int insertAuthUsers(String roleId, String[] userIds)
    {
        // 新增用户与角色管理
        List<IUserRole> list = new ArrayList<IUserRole>();
        for (String userId : userIds)
        {
            SysUserRole ur = new SysUserRole();
            ur.setUserRoleId(Uuids.getId());
            ur.setUserId(userId);
            ur.setRoleId(roleId);
            list.add(ur);
        }
        return userRoleMapper.batchUserRole(list);
    }
}
